// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009-2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_betarnd ( varargin )
    // Beta random numbers
    //
    // Calling Sequence
    //   R = distfun_betarnd ( a , b )
    //   R = distfun_betarnd ( a , b , [m,n] )
    //   R = distfun_betarnd ( a , b , m , n )
    //
    // Parameters
    //   a : a n-by-m matrix of doubles, the first shape parameter, a>0.
    //   b : a n-by-m matrix of doubles, the first shape parameter, b>0.
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers, in the interval [0,1].
    //
    // Description
    //   Generates random variables from the Beta distribution.
    //
    //   Any scalar input argument is expanded to a matrix of doubles of 
	//   the same size as the other input arguments.
    //
    // Examples
    //
    // // Set the seed so as to always get the same results.
    // distfun_seedset(1);
    //
    // // Use R = distfun_betarnd ( a , b )
    // distfun_betarnd(1:6,(1:6)^-1)
    // distfun_betarnd(1:6,1)
    // distfun_betarnd(1,(1:6)^-1)
    //
    // // Check mean and variance for R = distfun_betarnd ( a , b )
    // N = 1000;
    // a = 1:6;
    // b = (1:6)^-1;
    // for i = 1:N
    //   computed(i,1:6) = distfun_betarnd(a,b);
    // end
    // [M,V] = distfun_betastat ( a , b )
    // c = mean(computed(1:N,1:6) , "r" )
    // c = variance(computed(1:N,1:6) , "r" )
    //
    // // Check R = distfun_betarnd ( a , b , v )
    // computed = distfun_betarnd(2,1,[1 5])
    // computed = distfun_betarnd(2,1,[3 2])
    //
    // // Check mean and variance for R = distfun_betarnd ( a , b )
    // N = 1000;
    // a = 2;
    // b = 3;
    // computed = distfun_betarnd(2,3,[1 N]);
    // [M,V] = distfun_betastat ( a , b )
    // c = mean(computed(1:N) )
    // c = variance(computed(1:N) )
    //
    // // Check R = distfun_betarnd ( a , b , m , n )
    // computed = distfun_betarnd([1 2 3;4 5 6],0.1,2,3)
    // computed = distfun_betarnd(2,1,2,3)
    // computed = distfun_betarnd(1,[1 2 3;4 5 6],2,3)
    //
    // // Check mean and variance for R = distfun_betarnd ( a , b )
    // N = 1000;
    // a = 2;
    // b = 3;
    // computed = distfun_betarnd(a,b,1,N);
    // [M,V] = distfun_betastat ( a , b )
    // c = mean(computed(1:N) )
    // c = variance(computed(1:N) )
    //
    // // Make a plot of the actual distribution of the numbers
    // a = 2;
    // b = 3;
    // data = distfun_betarnd(a,b,1,1000);
    // histplot(10,data)
    // x = linspace(0,1,1000);
    // y = distfun_betapdf(x,a,b);
    // plot(x,y)
    // xtitle("Beta random variables","X","Density");
    // legend(["Empirical","PDF"]);
    //
    // Authors
    //   Copyright (C) 2012 - Michael Baudin
    //   Copyright (C) 2009-2011 - DIGITEO - Michael Baudin

    // Load Internals lib
    path = distfun_getpath (  )
    internallib  = lib(fullfile(path,"macros","internals"))

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_betarnd" , rhs , 2:4 )
    apifun_checklhs ( "distfun_betarnd" , lhs , 0:1 )
    //
    a = varargin(1)
    b = varargin(2)
    //
    // Check type
    apifun_checktype ( "distfun_betarnd" , a , "a" , 1 , "constant" )
    apifun_checktype ( "distfun_betarnd" , b , "b" , 2 , "constant" )
    if ( rhs == 3 ) then
        v = varargin(3)
    end
    if ( rhs == 4 ) then
        m = varargin(3)
        n = varargin(4)
    end
    //
    // Check size:  OK
    //
    // Check content
	tiniest=number_properties("tiniest")
    apifun_checkgreq ( "distfun_betarnd" , a , "a" , 1 , tiniest )
    apifun_checkgreq ( "distfun_betarnd" , b , "b" , 2 , tiniest )  
    //
    // Check v, m, n
    distfun_checkvmn ( "distfun_betarnd" , 3 , varargin(3:$) )
    //
    [a,b] = apifun_expandfromsize ( 2 , varargin(1:$) )
    m = size(a,"r")
    n = size(a,"c")
    if ( a==[] ) then
        R = []
        return
    end
    R = distfun_rndbeta(a,b,m,n) 
endfunction

