// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009-2010 - DIGITEO - Michael Baudin
// Copyright (C) 2009 - CEA - Jean-Marc Martinez
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

function C = distfun_cov ( varargin )
    // Returns the empirical covariance matrix.
    //
    // Calling Sequence
    //   C=distfun_cov(x)
    //   C=distfun_cov(x,0)
    //   C=distfun_cov(x,1)
    //   C=distfun_cov(x,y)
    //   C=distfun_cov(x,y,0)
    //   C=distfun_cov(x,y,1)
    //
    // Parameters
    // x: a matrix of doubles
    // y: a matrix of doubles
    // C: a square matrix of doubles, the empirical covariance
    //
    // Description
    // If x is a nobs-by-1 matrix, 
	// then distfun_cov(x) returns the variance of x, 
	// normalized by nobs-1. 
	//
    // If x is a nobs-by-nvar matrix, 
	// then distfun_cov(x) returns the nvar-by-nvar covariance matrix of the 
	// columns of x, normalized by nobs-1. 
	// Here, each column of x is a variable and 
	// each row of x is an observation.
	//
    // If x and y are two nobs-by-1 matrices, 
	// then distfun_cov(x,y) returns the 2-by-2 covariance matrix of x and 
	// y, normalized by nobs-1, where nobs is the number of observations.
	//
	// distfun_cov(x,0) is the same as distfun_cov(x) and 
	// distfun_cov(x,y,0) is the same as distfun_cov(x,y). 
	// In this case, if the population is from a normal distribution, 
	// then C is the best unbiased estimate of the covariance matrix. 
	//
	// distfun_cov(x,1) and distfun_cov(x,y,1) normalize by nobs.
	// In this case, C is the second moment matrix of the 
	// observations about their mean. 
	//
    // Examples
    // x = [1;2];
    // y = [3;4];
    // C=distfun_cov(x,y)
    // expected = [0.5,0.5;0.5,0.5]
    ////
    // x = [230;181;165;150;97;192;181;189;172;170];
    // y = [125;99;97;115;120;100;80;90;95;125];
    // expected = [
    // 1152.4556,-88.911111
    // -88.911111,244.26667
    // ]
    // C=distfun_cov(x,y)
    //
    // Authors
	// Copyright (C) 2012 - Michael Baudin
	// Copyright (C) 2009-2010 - DIGITEO - Michael Baudin
	// Copyright (C) 2009 - CEA - Jean-Marc Martinez
    //
    // Bibliography
    // "Introduction to probability and statistics for engineers and scientists.", Sheldon Ross

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_cov" , rhs , 1:3 )
    apifun_checklhs ( "distfun_cov" , lhs , 0:1 )
    //
    if (rhs==1) then
        x = varargin(1)
        //
        // Check type
        apifun_checktype ( "distfun_cov" , x , "x" , 1 , "constant" )
		nobs = size(x,"r")
		r = 1/(nobs-1)
		A = x
    elseif (rhs==2) then
		//
        x = varargin(1)
        y = varargin(2)
        //
        // Check type
        apifun_checktype ( "distfun_cov" , x , "x" , 1 , "constant" )
        apifun_checktype ( "distfun_cov" , y , "y" , 2 , "constant" )
        //
        // Check size
		nobs = size(x,"r")
		if (size(y,"*")==1) then
			apifun_checkoption ( "distfun_cov" , y , "y" , 2, [0 1])
			if (y==1) then
				r = 1/nobs
				A = x
			elseif (y==0) then
				r = 1/(nobs-1)
				A = x
			end
		else
			apifun_checkdims ( "distfun_cov" , x , "x" , 1, [nobs 1] )
			apifun_checkdims ( "distfun_cov" , y , "y" , 2, [nobs 1] )
			r = 1/(nobs-1)
			A = [x,y]
		end
    elseif (rhs==3) then
		//
        x = varargin(1)
        y = varargin(2)
		nrmlztn = varargin(3)
        //
        // Check type
        apifun_checktype ( "distfun_cov" , x , "x" , 1 , "constant" )
        apifun_checktype ( "distfun_cov" , y , "y" , 2 , "constant" )
        apifun_checktype ( "nrmlztn" , nrmlztn , "nrmlztn" , 3 , "constant" )
        //
        // Check size
        nobs = size(x,"r")
        apifun_checkdims ( "distfun_cov" , x , "x" , 1, [nobs 1] )
        apifun_checkdims ( "distfun_cov" , y , "y" , 2, [nobs 1] )
        apifun_checkscalar ( "distfun_cov" , nrmlztn , "nrmlztn" , 3)
		//
		// Check content
        apifun_checkoption ( "distfun_cov" , nrmlztn , "nrmlztn" , 3, [0 1])
		A = [x,y]
		if (nrmlztn==1) then
			r = 1/nobs
		else
			r = 1/(nobs-1)
		end
    end
	//
	// Compute with A in the general case
    nvar = size(A,"c")
    nobs = size(A,"r")
	for i = 1 : nvar
		A(:,i) = A(:,i) - mean(A(:,i))
	end
	C = zeros(nvar,nvar)
	for i = 1 : nvar
		C(i,i) = A(:,i)'*A(:,i)*r
		for j = i+1 : nvar
			C(i,j) = A(:,i)'*A(:,j)*r
			C(j,i) = C(i,j)
		end
	end
endfunction
