// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_exprnd ( varargin )
    // Exponential random numbers
    //
    // Calling Sequence
    //   R = distfun_exprnd ( mu )
    //   R = distfun_exprnd ( mu , [m,n] )
    //   R = distfun_exprnd ( mu , m , n )
    //
    // Parameters
    //   mu : a matrix of doubles, the average. mu>0
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers.
    //
    // Description
    //   Generates random variables from the Exponential distribution.
    //
    //   Any scalar input argument is expanded to a matrix of doubles of 
	//   the same size as the other input arguments.
    //
    // Examples
    // // Set the seed so as to always get the same results.
    // distfun_seedset(1);
    //
    // // Use R = distfun_exprnd ( mu )
    // distfun_exprnd(1:6)
    //
    // // Check mean and variance for R = distfun_exprnd ( mu )
    // N = 1000;
    // for mu = 1:6;
    //   computed = distfun_exprnd(mu,[1 N]);
    //   disp(mu)
    //   c = mean(computed)
    //   e = mu
    //   c = st_deviation(computed)
    //   e = mu
    // end
    //
    // // Check R = distfun_exprnd ( mu , v )
    // computed = distfun_exprnd(2,[1 5])
    // computed = distfun_exprnd(2,[3 2])
    //
    // // Check mean and variance for R = distfun_exprnd ( a , b )
    // N = 1000;
    // mu = 2;
    // computed = distfun_exprnd(mu,[1 N]);
    // c = mean(computed(1:N) )
    // e = mu
    // c = st_deviation(computed(1:N) )
    // e = mu
    //
    // // Check R = distfun_exprnd ( mu , m , n )
    // computed = distfun_exprnd([1 2 3;4 5 6],2,3)
    // computed = distfun_exprnd(2,2,3)
    // computed = distfun_exprnd(1,2,3)
    //
    // // Check mean and variance for R = distfun_exprnd ( mu )
    // N = 1000;
    // mu = 2;
    // computed = distfun_exprnd(mu,1,N);
    // c = mean(computed(1:N) )
    // e = mu
    // c = st_deviation(computed(1:N) )
    // e = mu
    //
    // // Make a plot of the actual distribution of the numbers
    // mu = 2;
    // data = distfun_exprnd(mu,1,1000);
    // scf();
    // histplot(10,data)
    // x = linspace(0,14,1000);
    // y = distfun_exppdf(x,mu);
    // plot(x,y)
    // xtitle("Exponential Random Numbers","X","Density")
    // legend(["Empirical","PDF"]);
    //
    // Authors
    //   Copyright (C) 2012 - Michael Baudin
    //   Copyright (C) 2009-2011 - DIGITEO - Michael Baudin

    // Load Internals lib
    path = distfun_getpath (  )
    internallib  = lib(fullfile(path,"macros","internals"))

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_exprnd" , rhs , 1:3 )
    apifun_checklhs ( "distfun_exprnd" , lhs , 0:1 )
    //
    mu = varargin(1)
    //
    // Check type
    apifun_checktype ( "distfun_exprnd" , mu , "mu" , 1 , "constant" )
    tiny = number_properties("tiny")
    apifun_checkgreq ( "distfun_exprnd" , mu , "mu" , 1 , tiny )
    if ( rhs == 2 ) then
        v = varargin(2)
    end
    if ( rhs == 3 ) then
        m = varargin(2)
        n = varargin(3)
    end
    //
    // Check v, m, n
    distfun_checkvmn ( "distfun_exprnd" , 2 , varargin(2:$) )
    //
    mu = apifun_expandfromsize ( 1 , varargin(1:$) )
    m = size(mu,"r")
    n = size(mu,"c")
	R=distfun_rndexp(mu,m,n)
endfunction

