// Copyright (C) 2012 - Prateek Papriwal
// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_frnd(varargin)
    // F-Distribution random numbers
    //
    // Calling Sequence
    //   R = distfun_frnd(v1,v2)
    //   R = distfun_frnd(v1,v2,[m,n])
    //   R = distfun_frnd(v1,v2,m,n)
    //
    // Parameters
    //   v1 : a n-by-m matrix of doubles, numerator degrees of freedom, v1>0 (can be non integer). 
    //   v2 : a n-by-m matrix of doubles, denominator degrees of freedom, v2>0 (can be non integer). 
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers in the set [0,∞).
    //
    // Description
    //   Generates random variables from the F-distribution function.
    //
    //   Any scalar input argument is expanded to a matrix of doubles 
    //   of the same size as the other input arguments.
    //
    // Examples
    // // set the initial seed for tests
    // distfun_seedset(1);
    // // Test with expanded v1 and v2
    // computed = distfun_frnd(1:6,1:6)
    // expected = [
	// 2.905302978312094830D-01 
	// 1.579862093854028371D-01 
	// 1.642623974391084429D+00 
	// 6.260081014837054481D-01 
	// 4.407037914058102857D-01 
	// 2.654717535342397405D-01
	// ]'
    //
    // // Check R = distfun_frnd(v1,v2,v)
    // computed = size(distfun_frnd(2,5,[3 3]))
    // expected = [3 3]
    //
    // // Check mean and variance
    // N = 5000;
    // v1 = 3;
    // v2 = 5;
    // R = distfun_frnd(v1,v2,[1 N]);
    // RM = mean(R)
    // RV = variance(R)
    // [M,V] = distfun_fstat(v1,v2)
    //
    // // Make a plot of the actual distribution of the numbers
    // v1 = 10;
    // v2 = 50;
    // data = distfun_frnd(v1,v2,1,1000);
    // scf();
    // histplot(20,data)
    // x = linspace(0,4,1000);
    // y = distfun_fpdf(x,v1,v2);
    // plot(x,y)
    // xtitle("F Random Numbers","X","Density")
    // legend(["Empirical","PDF"]);
	//
	// // Make a plot of a ratio of chi-square random numbers
	// v1 = 10;
	// v2 = 50;
	// R1 = distfun_chi2rnd(v1,1,1000);
	// R2 = distfun_chi2rnd(v2,1,1000);
	// data = (R1./v1)./(R2./v2);
	// scf();
	// histplot(20,data)
	// x = linspace(0,4,1000);
	// y = distfun_fpdf(x,v1,v2);
	// plot(x,y)
	// xtitle("$\textrm{Random Numbers }\frac{R_1/v_1}{R_2/v_2}$","X","Density")
	// legend(["Empirical","PDF"]);
    //
    // Bibliography
    // http://en.wikipedia.org/wiki/F-distribution
    // 
    // Authors
    // Copyright (C) 2012 - Prateek Papriwal
    // Copyright (C) 2012 - Michael Baudin
    path = distfun_getpath()
    internallib = lib(fullfile(path,"macros","internals"))

    [lhs,rhs] = argn()

    apifun_checkrhs("distfun_frnd",rhs,2:4)
    apifun_checklhs("distfun_frnd",lhs,0:1)
    
    v1 = varargin(1)
    v2=varargin(2)
    
    //
    // Check type
    apifun_checktype("distfun_frnd",v1,"v1",1,"constant")
    apifun_checktype("distfun_frnd",v2,"v2",2,"constant")
    
    // Check content 
    tiniest=number_properties("tiniest")
    apifun_checkgreq("distfun_frnd",v1,"v1",1,tiniest)
    apifun_checkgreq("distfun_frnd",v2,"v2",2,tiniest)
    
    if ( rhs == 3 ) then
        v = varargin(3)
    end
    if ( rhs == 4 ) then
        m = varargin(3)
        n = varargin(4)
    end
    //
    // Check v,m,n
    distfun_checkvmn ( "distfun_frnd" , 3 , varargin(3:$) )
    
    [v1,v2] = apifun_expandfromsize ( 2 , varargin(1:$) )

    if(v1 == []) then
        R = []
        return
    end

    m = size(v1,"r")
    n = size(v2,"c")

    R = distfun_rndf(v1,v2,m,n)
endfunction
