// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_gamrnd ( varargin )
    // Gamma random numbers
    //
    // Calling Sequence
    //   R = distfun_gamrnd ( a , b )
    //   R = distfun_gamrnd ( a , b , [m,n] )
    //   R = distfun_gamrnd ( a , b , m , n )
    //
    // Parameters
    //   a : a n-by-m matrix of doubles, the shape parameter, a>0.
    //   b : a n-by-m matrix of doubles, the scale parameter, b>0.
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the positive random numbers.
    //
    // Description
    //   Generates random variablesfrom the Gamma distribution function.
    //
    //   Any scalar input argument is expanded to a matrix of doubles of 
	//   the same size as the other input arguments.
    //
    // Examples
    // // Set the seed so as to always get the same results.
    // distfun_seedset(1);
    //
    // // Use R = distfun_gamrnd ( a , b )
    // distfun_gamrnd(1:6,(1:6)^-1)
    // distfun_gamrnd(1:6,1)
    // distfun_gamrnd(1,(1:6)^-1)
    //
    // // Check R = distfun_gamrnd ( a , b , v )
    // computed = distfun_gamrnd(2,1,[1 5])
    // computed = distfun_gamrnd(2,1,[3 2])
    //
    // // Check mean and variance for R = distfun_gamrnd ( a , b )
    // N = 1000;
    // a = 2;
    // b = 3;
    // [M,V] = distfun_gamstat ( a , b )
    // computed = distfun_gamrnd(2,3,[1 N]);
    // Mc = mean(computed)
    // Mv = variance(computed)
    //
    // // Check R = distfun_gamrnd ( a , b , m , n )
    // computed = distfun_gamrnd([1 2 3;4 5 6],0.1,2,3)
    // computed = distfun_gamrnd(2,1,2,3)
    // computed = distfun_gamrnd(1,[1 2 3;4 5 6],2,3)
    //
    // // Make a plot of the actual distribution of the numbers
    // a = 2;
    // b = 3;
    // data = distfun_gamrnd(a,b,1,1000);
    // scf();
    // histplot(10,data)
    // x = linspace(0,30,1000);
    // y = distfun_gampdf(x,a,b);
    // plot(x,y)
    // xtitle("Gamma Random Numbers","X","Density")
    // legend(["Empirical","PDF"]);
    //
    // Authors
    //   Copyright (C) 2012 - Michael Baudin
    //   Copyright (C) 2011 - DIGITEO - Michael Baudin

    // Load Internals lib
    path = distfun_getpath (  )
    internallib  = lib(fullfile(path,"macros","internals"))

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_gamrnd" , rhs , 2:4 )
    apifun_checklhs ( "distfun_gamrnd" , lhs , 0:1 )
    //
    a = varargin(1)
    b = varargin(2)
    //
    // Check type
    apifun_checktype ( "distfun_gamrnd" , a , "a" , 1 , "constant" )
    apifun_checktype ( "distfun_gamrnd" , b , "b" , 2 , "constant" )
    if ( rhs == 3 ) then
        v = varargin(3)
    end
    if ( rhs == 4 ) then
        m = varargin(3)
        n = varargin(4)
    end
    //
    // Check size : OK
    //
    // Check content
	tiny=number_properties("tiny")
    apifun_checkgreq( "distfun_gamrnd" , a , "a" , 1 , tiny )
    apifun_checkgreq( "distfun_gamrnd" , b , "b" , 2 , tiny )
    //
    // Check v, m, n
    distfun_checkvmn ( "distfun_gamrnd" , 3 , varargin(3:$) )
    //
    [a,b] = apifun_expandfromsize ( 2 , varargin(1:$) )
    //
    m = size(a,"r")
    n = size(a,"c")
    R = distfun_rndgam(a,b,m,n) 
endfunction

