// Copyright (C) 2012 - Prateek Papriwal
// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_hygernd(varargin)
    // Hypergeometric random numbers
    //
    // Calling Sequence
    //   R = distfun_hygernd(M,k,N)
    //   R = distfun_hygernd(M,k,N,[m,n])
    //   R = distfun_hygernd(M,k,N,m,n)
    //
    // Parameters
    //   M : a n-by-m matrix of doubles, the total size of the population . M belongs to the set {0,1,2,3........}
    //   k : a n-by-m matrix of doubles, the number of successful states in the population . k belongs to the set {0,1,2,3.......M-1,M}
    //   N : a n-by-m matrix of doubles, the total number of draws in the experiment . N belongs to the set {0,1,2,3,.......M-1,M}
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers in the set {0,1,2,...,min(N,k)}.
    //
    // Description
    //   Generates random variables from the Hypergeometric distribution function.
    //
    //   Any scalar input argument is expanded to a matrix of doubles 
    //   of the same size as the other input arguments.
	//
	// The algorithm is based on the inversion of the CDF.
    //
    // Examples
    // // Check R = distfun_hygernd(M,k,N,v)
    // computed = distfun_hygernd(80,50,30,[4 5])
    //
    // // Check mean and variance
    // M = 80;
    // k = 50;
    // N = 30;
    // n = 50000;
    // computed = distfun_hygernd(M,k,N,[1 n]);
    // c = mean(computed(1:n))
    // d = st_deviation(computed(1:n))
    // [M,V] = distfun_hygestat(M,k,N)
    //
    // // Check actual distribution
    // M=80;
    // k=50;
    // N=30;
    // nsamples=10000;
    // R=distfun_hygernd(M,k,N,1,nsamples);
    // h=scf();
    // distfun_inthisto(R);
    // h.children.children(1).children.background=-2;
    // [Me,Va] = distfun_hygestat(M,k,N);
    // x=int(Me-3*sqrt(Va)):int(Me+3*sqrt(Va));
    // y=distfun_hygepdf(x,M,k,N);
    // plot(x,y,"ro-");
    // xtitle("Hypergeometric Random Numbers","X","Frequency")
    // legend(["Empirical","PDF"]);
    //
    // Bibliography
    // http://en.wikipedia.org/wiki/Hypergeometric_distribution
    //
    // Authors
    // Copyright (C) 2012 - Prateek Papriwal
    // Copyright (C) 2012 - Michael Baudin

    path = distfun_getpath()
    internallib = lib(fullfile(path,"macros","internals"))

    [lhs,rhs] = argn()

    apifun_checkrhs("distfun_hygernd",rhs,3:5)
    apifun_checklhs("distfun_hygernd",lhs,0:1)

    M = varargin(1)
    k = varargin(2)
    N = varargin(3)
    //
    // Check type
    apifun_checktype ( "distfun_hygernd" , M , "M" , 1 , "constant" )
    apifun_checktype ( "distfun_hygernd" , k , "k" , 2 , "constant" )
    apifun_checktype ( "distfun_hygernd" , N , "N" , 3 , "constant" )
    //
    // Check content
    apifun_checkgreq("distfun_hygernd",M,"M",1,0)
    apifun_checkgreq("distfun_hygernd",k,"N",2,0)
    apifun_checkgreq("distfun_hygernd",N,"k",3,0)
    //
    apifun_checkflint("distfun_hygernd",M,"M",1)
    apifun_checkflint("distfun_hygernd",k,"k",2)
    apifun_checkflint("distfun_hygernd",N,"N",3)
    //
    if ( rhs == 4 ) then
        v = varargin(4)
    end
    if ( rhs == 5 ) then
        m = varargin(4)
        n = varargin(5)
    end
    //
    // Check v,m,n
    distfun_checkvmn ( "distfun_hygernd" , 4 , varargin(4:$) )
    //
    [M,k,N] = apifun_expandfromsize ( 3 , varargin(1:$) )
    if (M == []) then
        R = []
        return
    end
    //
    apifun_checkloweq("distfun_hygernd",k,"k",2,M) // k<=M
    apifun_checkloweq("distfun_hygernd",N,"N",3,M) // N<=M
    //
    m = size(M,"r")
    n = size(M,"c")
    R = distfun_rndhyge(M,k,N,m,n)
endfunction
