// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_normrnd ( varargin )
    // Normal random numbers
    //
    // Calling Sequence
    //   R = distfun_normrnd ( mu , sigma )
    //   R = distfun_normrnd ( mu , sigma , [m,n] )
    //   R = distfun_normrnd ( mu , sigma , m , n )
    //
    // Parameters
    //   mu : a n-by-m matrix of doubles, the average
    //   sigma : a n-by-m matrix of doubles, the standard deviation. sigma>0.
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers.
    //
    // Description
    //   Generates random variables from the Normal distribution function.
    //
    //   Any scalar input argument is expanded to a matrix of doubles of 
	//   the same size as the other input arguments.
    // 
    // Examples
    // // Set the seed so as to always get the same results.
    // distfun_seedset(1);
    //
    // // Test both mu and sigma expanded
    // computed = distfun_normrnd(1:6,(1:6)^-1)
    //
    // // Test sigma expansion
    // computed = distfun_normrnd(1:6,2.0)
    //
    // // Test mu expansion
    // computed = distfun_normrnd(1.0,1:6)
    //
    // // Test with v
    // computed = distfun_normrnd(0,1,[3 2])
    //
    // // Test with m, n
    // computed = distfun_normrnd(0,1,3,2)
    //
    // // Make a plot of the actual distribution of the numbers
    // mu = 2;
    // sigma = 3;
    // data = distfun_normrnd(mu,sigma,1,1000);
    // scf();
    // histplot(10,data)
    // x = linspace(-10,10,1000);
    // y = distfun_normpdf(x,mu,sigma);
    // plot(x,y)
    // xtitle("Normal Random Numbers","X","Density")
    // legend(["Empirical","PDF"]);
    //
    // Authors
    //   Copyright (C) 2009 - 2011 - DIGITEO - Michael Baudin

    // Load Internals lib
    path = distfun_getpath (  )
    internallib  = lib(fullfile(path,"macros","internals"))

    [lhs,rhs]=argn()
    apifun_checkrhs ( "distfun_normrnd" , rhs , 2:4 )
    apifun_checklhs ( "distfun_normrnd" , lhs , 0:1 )
    //
    mu = varargin(1)
    sigma = varargin(2)
    //
    // Check type
    apifun_checktype ( "distfun_normrnd" , mu , "mu" , 1 , "constant" )
    apifun_checktype ( "distfun_normrnd" , sigma , "sigma" , 2 , "constant" )
    if ( rhs == 3 ) then
        v = varargin(3)
    end
    if ( rhs == 4 ) then
        m = varargin(3)
        n = varargin(4)
    end
    //
    // Check size : OK
    //
    // Check content
    tiny = number_properties("tiny")
    apifun_checkgreq ( "distfun_normrnd" , sigma , "sigma" , 2 , tiny )
    //
    // Check v, m, n
    distfun_checkvmn ( "distfun_normrnd" , 3 , varargin(3:$) )
    //
    [mu,sigma] = apifun_expandfromsize ( 2 , varargin(1:$) )
    m = size(mu,"r")
    n = size(mu,"c")
    R = distfun_rndnorm(mu,sigma,m,n) 
endfunction

