// Copyright (C) 2012 - Prateek Papriwal
// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function R = distfun_trnd(varargin)
    // T random numbers
    //
    // Calling Sequence
    //   R = distfun_trnd ( v )
    //   R = distfun_trnd ( v , [m,n] )
    //   R = distfun_trnd ( v , m , n )
    //
    // Parameters
    //   v : a n-by-m matrix of doubles, the number of degrees of freedom, v>0. 
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers, in the interval [0,1].
    //
    // Description
    //   Generates random variables from the T distribution.
    //
    //   Any scalar input argument is expanded to a matrix of doubles 
	//   of the same size as the other input arguments.
    //
    // Examples
    // // Produce 6 random numbers from T distribution
    // // and degrees of freedom from 1 to 6
    // distfun_trnd(1:6)
    //
    // // Compare empirical distribution and density
    // v=12;
    // N=10000;
    // R=distfun_trnd(v,1,N);
    // h=scf();
    // histplot(40,R);
    // h.children.children(1).children.background=-2;
    // [M,V]=distfun_tstat(v);
    // x=linspace(M-3*sqrt(V),M+3*sqrt(V),1000);
    // y=distfun_tpdf(x,v);
    // plot(x,y,"r-");
    // xtitle("T Random Numbers","X","Frequency")
    // legend(["Empirical","Density"]);
    //
    // Authors
    // Copyright (C) 2012 - Prateek Papriwal
    // Copyright (C) 2012 - Michael Baudin

    path = distfun_getpath()
    internallib = lib(fullfile(path,"macros","internals"))

    [lhs,rhs] = argn()

    apifun_checkrhs("distfun_trnd",rhs,1:3)
    apifun_checklhs("distfun_trnd",lhs,0:1)

    df = varargin(1)

    //
    // Check type
    apifun_checktype("distfun_trnd",df,"v",1,"constant")
    // Check content 
    tiny=number_properties("tiny") 
    apifun_checkgreq("distfun_trnd",df,"v",1,tiny)

    if ( rhs == 2 ) then
        v = varargin(2)
    end
    if ( rhs == 3 ) then
        m = varargin(2)
        n = varargin(3)
    end
    //
    // Check v,m,n
    distfun_checkvmn ( "distfun_trnd" , 2 , varargin(2:$) )

    df = apifun_expandfromsize ( 1 , varargin(1:$) )

    if(df == []) then
        R = []
        return
    end

    m = size(df,"r")
    n = size(df,"c")

    R = distfun_rndt(df,m,n)
endfunction
