/*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 2011 - DIGITEO - Michael Baudin
* Copyright (C) 2008 - INRIA
* Copyright (C) 2012 - Prateek Papriwal
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#ifndef _CDFLIB_H_
#define _CDFLIB_H_

#ifdef _MSC_VER
#if LIBCDFLIB_EXPORTS 
#define CDFLIB_IMPORTEXPORT __declspec (dllexport)
#else
#define CDFLIB_IMPORTEXPORT __declspec (dllimport)
#endif
#else
#define CDFLIB_IMPORTEXPORT
#endif

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS

//////////////////////////////////////////////////////
//
// All the function below have the same behavior.
// They return a integer status which says if the computation 
// performed well : 
//  * returns CDFLIB_OK if the function can be computed.
//  * returns CDFLIB_ERROR in case of error.
//
// This is the status of a function which performs correctly.
static int CDFLIB_OK = 0;

// The state of a function which generates an error.
static int CDFLIB_ERROR = 1;

//////////////////////////////////////////////////////
// To compute P(X<=x)
static int CDFLIB_LOWERTAIL = 1;

// To compute P(X>x)
static int CDFLIB_UPPERTAIL = 0;

//////////////////////////////////////////////////////
// To compute log(f(x))
static int CDFLIB_LOGCOMPUTE = 1;

// To compute f(x)
static int CDFLIB_LOGNOT = 0;
////////////////////////////////////

// Configure the function which prints messages.
// This function must be called before calling any 
// function.
void cdflib_messageSetFunction ( void (* f)(char * message) );

// Configure the function which generates random numbers uniform in [0,1).
// This function must be called at least once.
void cdflib_randSetFunction ( double (* f)(void) );

// Configure the function which generates random integers uniform in [a,b].
// This function must be called before at least once.
void cdflib_randIntegerInRangeSetFunction ( double (* f)(double a, double b) );

// Drivers for CDF and quantile functions
// Examples
/*
double p=0.1;
double q=0.9;
double x;
int status;
double expected;
status = cdflib_cdfnbn(2, &p, &q, &x, 50, 0.7, 0.3);
expected = 14.069628337395005956978;
*/
int cdflib_cdfnbn(int which, double *p, double *q, double *x, double n, double pr, double ompr);

// Examples
/*
double p=0.1;
double q=0.9;
double x;
int status;
double expected;
status = cdflib_cdffnc(2, &p, &q, &x, 4, 12, 0.3);
expected = 0.2765291823460812037361;
*/
int cdflib_cdffnc(int which, double *p, double *q, double *x, double dfn, double dfd, double phonc);

// Examples
/*
double p=0.1;
double q=0.9;
double x;
int status;
double expected;
status = cdflib_cdfchn(2, &p, &q, &x, 4, 0.3);
expected = 1.1458702849964366166091;
*/
int cdflib_cdfchn(int which, double *p, double *q, double *x, double df, double pnonc);

//////////////////////////////////////////////////////
// To compute enable verbose mode
static int CDFLIB_VERBOSEOFF = 0;

// To compute disable verbose mode
static int CDFLIB_VERBOSEON = 1;

// Configure the verbose mode.
void cdflib_verboseset(int verbosemode);

/////////////////////////////////////////////////////////////////////////
//
// Hypergeometric Distribution
//

// Computes the Hypergeometric PDF
//
// Input:
//   x : the number of successful draws in the experiment. x belongs to the set [0,min(k,N)]
//   M : the total size of the population. M belongs to the set {0,1,2,3........}
//   k : the number of successful states in the population. k belongs to the set {0,1,2,3,.......M-1,M}
//   N : the total number of draws in the experiment. N belongs to the set {0,1,2,3.......M-1,M}
// Output:
//   p, output : the probability P(X=x).
//
// Description
//   The function definition is:
//
//f(x,M,k,N) = \frac{\binom{k}{x} \binom{M-k}{N-x}}{\binom{M}{N}}
//
// with the convention that $\binom{M-k}{N-x}$ is zero if <literal>N-x > M-k</literal>.
//
// From the definition of the PDF, the input arguments must be so that:
// x<=N
// x<=k
// k<=M
// N<=M
//
// If one of these conditions is not satisfied, 
// an error is generated.
//
// The implementation uses the fact that:
//
// f(x)=binomialpdf(x,k,p)*binomialpdf(N-x,M-k,p)/binomialpdf(N,M,p)
//
// with p=N/M.
//
// Examples
/* 
int status;
double y;
status = cdflib_hygepdf(20,80,50,30,&y);
expected = 0.1596136;
*/
//
// References
// http://svn.r-project.org/R/trunk/src/nmath/dhyper.c
int cdflib_hygepdf(double x, double M, double k, double N, double *p);
int cdflib_hygeCheckX(char * fname, double x, double M, double k, double N);
int cdflib_hygeCheckParams(char * fname, double M, double k, double N);

// Computes the hypergeometric CDF
//
// Input:
//   lowertail : the tail. 
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//
// Examples
/* 
int status;
double y;
status = cdflib_hygecdf(20,80,50,30,CDFLIB_LOWERTAIL,&y);
expected = 0.7974774;
*/
int cdflib_hygecdf(double x, double M, double k, double N, int lowertail, double *p);

// Computes the inverse hypergeometric CDF
//
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//   lowertail : the tail. 
// Output:
//   x : the number of successful draws in the experiment. x belongs to the set [0,min(k,N)]
//
// Examples
/*
status = cdflib_hygeinvcdf(0.2,80,50,30,CDFLIB_LOWERTAIL,&y)
expected = 17
*/
int cdflib_hygeinvcdf(double p, double M, double k, double N, int lowertail, double *x);

// Returns hypergeometric outcomes
//
// Output:
//   x : the number of successful draws in the experiment. x belongs to the set [0,min(k,N)]
//
// Description
// Invert the hypergeometric CDF.
//
// Examples
/* 
double x;
int status;
status = cdflib_hygeinvcdf(80,50,30,&x);
expected = 17
*/
int cdflib_hygernd (double M, double k, double N, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Gamma Distribution
//

// Gamma PDF
//
// Parameters
// Input:
//   x : the outcome, x>=0
//   a : the shape parameter, a>0.
//   b : the scale parameter, b>0.
// Output:
//   y : the density
//
// Description
//   Computes the Gamma probability distribution function.
//
//   The function definition is:
//
//f(x,a,b) = \frac{1}{b^a\Gamma(a)} x^{a-1} \exp\left(-\frac{x}{b}\right)
//
// Compatibility note. 
// 
// This function is compatible with Matlab, but 
// not compatible with R. 
// Indeed, notice that b, the scale, is the inverse of the rate. 
// Other computing languages (including R), use 1/b as the 
// second parameter of the Gamma distribution.
//
// Examples
/* 
int status;
double y;
double expected;
status = distfun_gampdf(1,1,5,&y);
expected = 1.637461506155963586D-01;
*/
int cdflib_gammapdf( double x , double a , double b , double * y );

// Gamma CDF
//
// Parameters
// Input:
//   x : the outcome
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/*
int status;
double p;
double expected;
status = cdflib_gamcdf(1,2,3,CDFLIB_LOWERTAIL,&p);
expected = 0.0446249192349476506148;
*/
int cdflib_gamcdf(double x, double a, double b, int lowertail, double *p );

// Gamma Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome
//
// Examples
/*
int status;
double x;
double expected;
status = cdflib_gaminv(0.5,2,3,CDFLIB_LOWERTAIL,&x);
expected = 5.0350409700499820786490;
*/
int cdflib_gaminv(double p, double a, double b, int lowertail, double *x);

// Generate Gamma Random Variables
// Method
/*     Renames cdflib_sgamma from TOMS as slightly modified by BWB to use RANF */
/*     instead of SUNIF. */
/* Case A >= 1.0: */
/*     Ahrens, J.H. and Dieter, U. */
/*     Generating Gamma Variates by a */
/*     Modified Rejection Technique. */
/*     Comm. ACM, 25,1 (Jan. 1982), 47 - 54. */
/*     Algorithm GD */
/*     JJV altered the following to reflect cdflib_sgamma argument ranges */
/* Case 0.0 < A < 1.0: */
/*     Ahrens, J.H. and Dieter, U. */
/*     Computer Methods for Sampling from Gamma, */
/*     Beta, Poisson and Binomial Distributions. */
/*     Computing, 12 (1974), 223-246/ */
/*     Adapted algorithm GS. */
int cdflib_gamrnd(double a, double b, double *x);

/////////////////////////////////////////////////////////////////////////
//
// F Distribution
//

// F PDF
//   
// Parameters
// Input:
//   x : x is real and x>=0.
//   v1 : numerator degrees of freendom, v1>0. 
//   v2 : denominator degrees of freendom, v2>0.
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the f distribution function.
//   
//   Any scalar input argument is expanded to a matrix of doubles 
//   of the same size as the other input arguments.
//
// The F distribution has density
//
// f(x) = \frac{\sqrt{\frac{(v_1 x)^{v_1} v_2^{v_2}}{(v_1 x+v_2)^{v_1+v_2}}}}{x B\left(\frac{v_1}{2},\frac{v_2}{2}\right)}
//
// Examples
/*
int status;
double y;
double expected;
status = cdflib_fpdf(3,4,5,&y);
expected = 0.0681795538575894199962;
*/
//
// Bibliography
// Catherine Loader, http://svn.r-project.org/R/trunk/src/nmath/df.c 
//
int cdflib_fpdf(double x,double v1,double v2, double * y);

// F CDF
//   
// Parameters
// Input:
//   x : x is real and x>=0.
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_fcdf(3,4,5,CDFLIB_LOWERTAIL,&p);
expected = 0.8702965153994906000179;
*/
int cdflib_fcdf(double x, double v1, double v2, int lowertail, double *p);

// F Inverse CDF
//   
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : x is real and x>=0.
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_finv(0.7,4,5,CDFLIB_LOWERTAIL,&x);
expected = 1.6286329978911637805794;
*/
int cdflib_finv(double p, double v1, double v2, int lowertail, double *x);

// Generate F Random Variables
//
// Method
/*     Directly generates ratio of chisquare variates */
int cdflib_frnd(double v1, double v2, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Binomial Distribution
//

// Binomial PDF
//
// Parameters
// Input:
//   x : the number of Bernoulli trials after in which success occurs . x belongs to the set {0,1,2,3,...,N}. 
//       x has integer value.
//   n : the total number of binomial trials . N belongs to the set {1,2,3,4,.......}
//       n has integer value.
//   pr : the probability of success in a Bernoulli trial 
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the Binomial distribution function.
//   
//   Any scalar input argument is expanded to a matrix of doubles 
//   of the same size as the other input arguments.
//
//    The function definition is:
//
//f(x,n,pr) = \binom{n}{x} p_r^x (1-p_r)^{n-x}
//
//Examples
/* 
int status;
double y;
double expected;
status = cdflib_binopdf(3,200,0.02,&y);
expected = 0.1963468373887976292647;
*/
int cdflib_binopdf(double x, double n, double p, double * y);

// Binomial CDF
//
// Parameters
// Input:
//   x : the number of Bernoulli trials after in which success occurs . x belongs to the set {0,1,2,3,...,N}. 
//       x has integer value.
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
//Examples
/* 
int status;
double p;
double expected;
status = cdflib_binocdf(3,200,0.02,CDFLIB_LOWERTAIL,&p);
expected = 0.4314949731615046468747;
*/
int cdflib_binocdf(double x, double n, double pr, int lowertail, double *p);

// Binomial Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the number of Bernoulli trials after in which success occurs . x belongs to the set {0,1,2,3,...,N}. 
//       x has integer value.
//
//Examples
/* 
int status;
double x;
double expected;
status = cdflib_binoinv(0.9,200,0.02,CDFLIB_LOWERTAIL,&x);
expected = 7.;
*/
int cdflib_binoinv(double p, double n, double pr, int lowertail, double *x);

// Generate Binomial Random Variables
/*
If n<=2147483647, uses :
Method
This is algorithm BTPE from:
Kachitvichyanukul, V. and Schmeiser, B. W.
Binomial Random Variate Generation.
Communications of the ACM, 31, 2
(February, 1988) 216.

If n is larger, invert the CDF.
*/
int cdflib_binornd (double n, double pr, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Normal Distribution
//

// Normal PDF
//
// Parameters
// Input:
//   x : the outcome
//   mu : the mean 
//   sigma : the standard deviation 
// Output:
//   y : the density
//
// Description
//   Computes the probability distribution function 
// of the Normal (Laplace-Gauss) function.
//
//   The function definition is:
//
//f(x,\mu,\sigma) = \frac{1}{\sigma\sqrt{2\pi}} \exp\left(\frac{-(x-\mu)^2}{2\sigma^2}\right)
//
// Examples
/* 
int status;
double y;
double expected;
status = cdflib_normpdf ( 2 , 1.0 , 2.0, &y );
expected = 0.176032663382150;
*/
int cdflib_normpdf(double x, double mu, double sigma, double *y);

// Normal CDF
//
// Parameters
// Input:
//   x : the outcome
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_normcdf ( 2 , 1.0 , 2.0, CDFLIB_LOWERTAIL, &p );
expected = 0.6914624612740130071842;
*/
int cdflib_normcdf(double x, double mu, double sigma, int lowertail, double *p);

// Normal Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_norminv ( 0.7 , 1.0 , 2.0, CDFLIB_LOWERTAIL, &x );
expected = 2.0488010254160813339297;
*/
int cdflib_norminv(double p, double mu, double sigma, int lowertail, double *x);

// Generate Normal Random Variables
// Method
/*     Renames cdflib_snorm from TOMS as slightly modified by BWB to use RANF */
/*     instead of SUNIF. */
/*     Ahrens, J.H. and Dieter, U. */
/*     Extensions of Forsythe's Method for Random */
/*     Sampling from the Normal Distribution. */
/*     Math. Comput., 27,124 (Oct. 1973), 927 - 937. */
int cdflib_normrnd (double mu, double sigma, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Uniform Distribution
//

// Computes the Uniform PDF
//
// Parameters
// Input:
//   x : the outcome
//   a : the lower bound
//   b : the upper bound
// Output:
//   y : the density.
//
// Description
//   Computes the probability distribution function of 
//   the Uniform distribution function.
//   
//   Any scalar input argument is expanded to a matrix of doubles 
//   of the same size as the other input arguments.
//
//   The function definition is:
//
//f(x,a,b) = 1/(b-a), if x\in[a,b],
//           0,       otherwise.
//
// Examples
// int status;
// double y;
//  status = cdflib_unifpdf ( 4, 3, 6, &y );
//  yexpected = 0.33333;
//
int cdflib_unifpdf(double x, double a, double b, double *y);
int cdflib_unifCheckX(char * fname , double x);
int cdflib_unifCheckParams(char * fname , double a, double b);

// Computes the Uniform CDF
//
// Input:
//   lowertail : the tail. 
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//
// Examples
// int status;
// double y;
//  status = cdflib_unifcdf ( 2, 1, 5, CDFLIB_LOWERTAIL, &y );
//  pexpected = 0.25;
//
int cdflib_unifcdf(double x, double a, double b, int lowertail, double *p);

// Computes the Uniform Inverse CDF
//
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL.
//   lowertail : the tail. 
// Output:
//   x : the outcome.
//
// Examples
// int status;
// double x;
//  status = cdflib_unifinv ( 0.7, 1., 2., CDFLIB_LOWERTAIL, &x );
//  xexpected = 1.7;
//
int cdflib_unifinv(double p, double a, double b, int lowertail, double *x);

// Generate Uniform Random Variables
//
/*               GeNerate Uniform DOUBLE PRECISION between LOW and HIGH */
// Function
/*     Generates a DOUBLE PRECISION uniformly distributed between LOW and HIGH. */
// Arguments
/*     a : Low bound (exclusive) */
/*     b : High bound (exclusive) */
int cdflib_unifrnd(double a, double b, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Chi-Square Distribution
//

// Chi-Square PDF
//   
// Parameters
// Input:
//   x : the outcome, x>=0
//   k : the number of degrees of freedom. k>0 (can be non-integer)
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the Chi-square distribution function.
//
//   The function definition is:
//
//f(x,k) = \frac{1}{2^{\frac{k}{2}}\Gamma\left(\frac{k}{2}\right)} x^{\frac{k}{2}-1} \exp\left(-\frac{x}{2}\right)
//
// Examples
/* 
int status;
double y;
double expected;
status = cdflib_chi2pdf(4,5,&y);
expected = 0.1439759107018347694673;
*/ 
// Bibliography
// http://en.wikipedia.org/wiki/Chi-squared_distribution
int cdflib_chi2pdf(double x, double k, double *y);

// Chi-Square CDF
//   
// Parameters
// Input:
//   x : the outcome, x>=0
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_chi2cdf(4,5,CDFLIB_LOWERTAIL,&p);
expected = 0.4505840486472180161925;
*/ 
int cdflib_chi2cdf(double x, double k, int lowertail, double *p);

// Chi-Square Inverse CDF
//   
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome, x>=0
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_chi2inv(0.7,5,CDFLIB_LOWERTAIL,&x);
expected = 6.0644299841548976459649;
*/ 
int cdflib_chi2inv(double p, double k, int lowertail, double *x);

// Generate Chi Random Variables
//
// Arguments
/*     k : Degrees of freedom of the chisquare, k>0 */
// Method
/*     Uses relation between chisquare and gamma. */
int cdflib_chi2rnd (double k, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Geometric Distribution
//

// Geometric PDF
//   
// Parameters
// Input:
//   x : the number of Bernoulli trials after which the first success occurs. x belongs to the set {0,1,2,3,.....}
//       x must have an integer value. If not, an error is generated.
//   pr : the probability of success in a Bernoulli trial. pr in (0,1]
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the Geometric distribution function.
//   
//   The function definition is:
//
//f(x,pr) = pr\left(1-pr\right)^x
//
// Compatibility Note : x belongs to the set {0,1,2,3,...}. 
// This choice is compatible with Matlab and R. 
// This is different from Scilab v5 grand(m,n,"geom"), which 
// uses {1,2,3,...}. 
//
// Examples
// int status;
// double y;
// status = cdflib_geopdf(3,0.5,&y);
// yexpected = 0.0625;
//
// Bibliography
// http://en.wikipedia.org/wiki/Geometric_distribution
int cdflib_geopdf(double x, double pr, double *y);
int cdflib_geoCheckParams(char * fname, double pr);
int cdflib_geoCheckX(char * fname, double x);

// Geometric CDF
//
// Parameters
// Input:
//     lowertail: the tail.
// Output:
//     p: the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
// int status;
// double p;
// status = cdflib_geocdf(3,0.5, CDFLIB_LOWERTAIL, &p);
// expected = 0.9375;
//
int cdflib_geocdf(double x, double pr, int lowertail, double *p);

// Geometric InverseCDF
//
// Parameters
// Input:
//     p: the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//     lowertail: the tail.
// Output:
//     x: the outcome
//
// Examples
// int status;
// double x;
// status = cdflib_geoinv(0.999,0.5, CDFLIB_LOWERTAIL, &x);
// xexpected = 9;
//
int cdflib_geoinv(double p, double pr, int lowertail, double *x);

// Generate Geometric Random Variables
/*
*  METHOD 
*     inversion of the cdf leads to :
*
*     (1)   X = floor( log(1-u) / log(1-p) )
*
*     u being a random deviate from U[0,1).
*
*     by taking into account that 1-u follows also U(0,1)) this may be 
*     replaced with X = ceil( log(u) / log(1-p) )-1 or floor(log(u)/log(1-p))
*     which needs less work. But as ranf() provides number in [0,1[ , 0 may be 
*     gotten and these formulae may give then +oo.
* 
*/
int cdflib_geornd(double pr, double *x);

/////////////////////////////////////////////////////////////////////////
//
// T Distribution
//

// T PDF
//
// Parameters
// Input:
//   x : the outcome.
//   v : the number of degrees of freedom, v>0. 
// Output:
//   y : the density
//
// Description
//   Computes the T probability distribution function.
//
// The function definition is:
//
//f(x,v) = \frac{\Gamma\left(\frac{v+1}{2}\right)}{\Gamma\left(\frac{v}{2}\right)} \frac{1}{\sqrt{v\pi}} \frac{1}{\left(1+\frac{x^2}{v}\right)^{\frac{v+1}{2}}}
//
// Examples
/* 
int status;
double y;
double expected;
status = cdflib_tpdf(-3, 2, &y);
expected = 0.0274101222343421414840;
*/
int cdflib_tpdf(double x, double v, double *y);

// T CDF
//
// Parameters
// Input:
//   x : the outcome.
// Output:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/* 
int status;
double p;
double expected;
status = cdflib_tcdf(-3, 2, CDFLIB_LOWERTAIL, &p);
expected = 0.0477329831333545562266;
*/
int cdflib_tcdf(double x, double v, int lowertail, double *p);

// T Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome.
//
// Examples
/* 
int status;
double x;
double expected;
status = cdflib_tinv(0.7, 2, CDFLIB_LOWERTAIL, &x);
expected = 0.6172133998483675387803;
*/
int cdflib_tinv(double p, double v, int lowertail, double *x);

// Generate T Random Variables
// Function
/*     Generates a single random deviate from a T distribution */
// Arguments
/*     v : the number of degrees of freedom, v>0. */
int cdflib_trnd(double v, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Poisson Distribution
//

// Poisson PDF
//   
// Parameters
// Input:
//   x : the number of occurrences of events. x belongs to the set {0,1,2,3,.....}.
//       x must have an integer value.
//   lambda : the average rate of occurrence. We must have lambda>0.
// Output:
//   y : the probability density.
//
// Description
//   Computes the probability distribution function of 
//   the poisson distribution function.
//
//   The function definition is:
//
//f(x,\lambda) = \frac{\lambda^x e^{-\lambda}}{x!}
//
// Note : x belongs to the set {0,1,2,3,...}.
//
// Examples
/*
int status;
double y;
double expected;
status = cdflib_poisspdf(0,2,&y);
expected = 0.1353352832366127023178;
*/
// Bibliography
// http://en.wikipedia.org/wiki/Poisson_distribution
int cdflib_poisspdf(double x, double lambda, double *y);
int cdflib_poissCheckX(char * fname, double x);
int cdflib_poissCheckParams(char * fname, double lambda);

int cdfpoi_(int which, double *p, double *q, double *x, double lambda);

// Poisson CDF
//   
// Parameters
// Input:
//   x : the outcome
// Output:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/*
int status;
double p;
double expected;
status = cdflib_poisscdf(1,2,CDFLIB_LOWERTAIL,&p);
expected = 0.4060058497098381069534;
*/
int cdflib_poisscdf(double x, double lambda, int lowertail, double *p);

// Poisson Inverse CDF
//   
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome
//
// Examples
/*
int status;
double x;
double expected;
status = cdflib_poissinv(0.7,2,CDFLIB_LOWERTAIL,&x);
expected = 3.;
*/
int cdflib_poissinv(double p, double lambda, int lowertail, double *x);

// Generate Poisson Random Variables
/*
Method
Renames KPOIS from TOMS as slightly modified by BWB to use RANF
instead of SUNIF.

References
AHRENS, J.H. AND DIETER, U.
COMPUTER GENERATION OF POISSON DEVIATES
FROM MODIFIED NORMAL DISTRIBUTIONS.
ACM TRANS. MATH. SOFTWARE, 8,2 (JUNE 1982), 163 - 179.
(SLIGHTLY MODIFIED VERSION OF THE PROGRAM IN THE ABOVE ARTICLE)
*/
int cdflib_poissrnd (double lambda, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Log-Normal Distribution
//

// Log-Normal PDF
//
// Parameters
// Input:
//     x: the outcome
//     mu: the mean of the underlying normal variable.
//     sigma: the variance of the underlying normal variable, sigma>0.
// Output:
//     y: the density
//
// Description
//   This function computes the Lognormal PDF.
//
//   Any scalar input argument is expanded to a matrix of doubles of the same size 
//   as the other input arguments.
//
// The Lognormal distribution with parameters mu and sigma has density
//
// f(x) = \frac{1}{\sigma x \sqrt{2 \pi}} \exp\left(-\frac{(\ln(x) - \mu)^2}{2 \sigma^2}\right), 
// if x> 0, and zero otherwise.
//
// Examples
// int status;
// double y;
// status = cdflib_lognpdf ( 2 , 0.0 , 1.0 , &y);
// expected = 0.156874019278981119;
//
// Bibliography
// Dider Pelat, "Bases et mthodes pour le traitement de donnes", section 8.2.8, "Loi log-normale".
// Wikipedia, Lognormal distribution, http://en.wikipedia.org/wiki/Log-normal_distribution
int cdflib_lognpdf(double x, double mu, double sigma, double *y);
int cdflib_lognCheckParams(char * fname, double mu, double sigma);
int cdflib_lognCheckX(char * fname, double x);

// Log-Normal CDF
//
// Parameters
// Input:
//     x: the outcome
// Output:
//     p: the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
// int status;
// double p;
// status = cdflib_logncdf(2., 3., 10., CDFLIB_LOWERTAIL, &p);
// expected = 0.4087797;
int cdflib_logncdf(double x, double mu, double sigma, int lowertail, double *p);

// Log-Normal Inverse CDF
//
// Parameters
// Input:
//     p: the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//     x: the outcome
//
// Examples
// int status;
// double x;
// status = cdflib_logninv(0.6, 1., 2., CDFLIB_LOWERTAIL, &x);
// expected = 4.5117910634839439865;
int cdflib_logninv(double p, double mu, double sigma, int lowertail, double *x);

// Generate Log-Normal Random Variables
// Function
/*     Generates a single random deviate from a log-normal distribution */
// Arguments
/*     mu : the mean of the underlying normal variable. */
/*     sigma : the standard deviation of the underlying normal variable. sigma>0. */
int cdflib_lognrnd(double mu, double sigma, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Exponential Distribution
//

// Exponential PDF
//
// Parameters
// Input:
//     x: the outcome
//     mu : the average. mu>0
// Output:
//     y: the density
//
// Description
//   This function computes the Exponential PDF.
//
//   Any scalar input argument is expanded to a matrix of doubles of the same size 
//   as the other input arguments.
//
// Any optional input argument equal to the empty matrix will be set to its
// default value.
//
// The exponential distribution with average mu has density
//
// f(x) = \frac{1}{\mu} e^{\frac{-x}{\mu}}
//
// for x >= 0 and is zero if x<0.
//
// Compatibility note. 
// 
// Notice that mu, the average, is the inverse of the rate. 
// Other computing languages (including R), use 1/mu as the 
// parameter of the exponential distribution.
//
// Examples
// double y;
// int status;
// status=cdflib_exppdf(3,2,&y);
// expected=0.111565080074214909;
//
// Bibliography
// Wikipedia, Exponential distribution function, http://en.wikipedia.org/wiki/Exponential_distribution
int cdflib_exppdf(double x, double mu, double *y);
int cdflib_expCheckX(char * fname, double x);
int cdflib_expCheckParams(char * fname, double mu);

// Exponential CDF
//
// Parameters
// Input:
//     x: the outcome
// Output:
//     p: the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
// double p;
// int status;
// status=cdflib_expcdf(2.,1./3.,CDFLIB_LOWERTAIL,&p);
// expected=9.975212478233337e-1;
//
int cdflib_expcdf(double x, double mu, int lowertail, double *p);

// Exponential Inverse CDF
//
// Parameters
// Input:
//     p: the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//     x: the outcome
//
// Examples
// double x;
// int status;
// status=cdflib_expinv(0.7,2.,CDFLIB_LOWERTAIL,&x);
// expected=2.4079456;
//
int cdflib_expinv(double p, double mu, int lowertail, double *x);

// Generate Exponential Random Variables
//
// Arguments
/*     mu : The mean of the exponential distribution from which */
/*            a random deviate is to be generated. (mu >= 0) */
// Method
/* Ahrens, J.H. and Dieter, U. */
/* Computer Methods for Sampling From the */
/* Exponential and Normal Distributions. */
/* Comm. ACM, 15,10 (Oct. 1972), 873 - 882. */
int cdflib_exprnd (double mu, double *x);

/////////////////////////////////////////////////////////////////////////
//
// Beta Distribution
//

// Beta PDF
//
// Parameters
// Input:
//   x : the outcome. Should be in the [0,1] interval.
//   a : the first shape parameter. a > 0
//   b : the second shape parameter. b > 0
// Output:
//   y : the density
//
// Description
//   Computes the Beta probability distribution function.
//
// Let us denote by B the Beta function. The function definition is:
//
//f(x,a,b) = \frac{1}{B(a,b)} x^{a-1} (1-x)^{b-1}
//
// if x is in [0,1]. 
//
// Examples
/*
int status;
double y;
double expected;
status = cdflib_betapdf ( 0.1 , 2 , 3 , &y);
expected = 0.9720;
*/ 
// Bibliography
// Catherine Loader, http://svn.r-project.org/R/trunk/src/nmath/dbeta.c
//
int cdflib_betapdf(double x, double a, double b, double *y);

// Beta CDF
//
// Parameters
// Input:
//   x : the outcome, x in [0,1]
// Output:
//   p : the probability, P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
//
// Examples
/*
int status;
double p;
double expected;
status = cdflib_betacdf( 0.1, 2, 3, CDFLIB_LOWERTAIL, &p);
expected = 0.0523;
 */
int cdflib_betacdf(double x, double a, double b, int lowertail, double *p);

// Beta Inverse CDF
//
// Parameters
// Input:
//   p : the probability, p in [0,1], P(X<=x) if lowertail=CDFLIB_LOWERTAIL, 
//                       P(X>x) if lowertail=CDFLIB_UPPERTAIL
// Output:
//   x : the outcome. 
//
// Examples
/*
int status;
double x;
double expected;
status = cdflib_betainv ( 0.5, 2, 3, CDFLIB_LOWERTAIL, &x);
expected = 0.3857275681323895644148;
 */
int cdflib_betainv(double p, double a, double b, int lowertail, double *x);

// Generate Beta Random Variables
/*
Function
Returns aa single random deviate from the beta distribution with
parameters A and B.  The density of the beta is
x^(aa-1) * (1-x)^(bb-1) / B(aa,bb) for 0 < x < 1
Arguments
A : First parameter of the beta distribution (A > 0)
B : Second parameter of the beta distribution (B > 0)
Method
R. C. H. Cheng
Generating Beta Variatew with Nonintegral Shape Parameters
Communications of the ACM, 21:317-322  (1978)
(Algorithms BB and BC)
*/
int cdflib_betarnd (double a, double b, double *x);

//////////////////////////////////////////////////////////////////////

// Generate Non-Central Chi Random Variables
// Function
/*     Generates random deviate  from the  distribution  of a  noncentral */
/*     chisquare with k degrees  of freedom and noncentrality  parameter */
/*     XNONC. */
// Arguments
/*     k : Degrees of freedom of the chisquare (k >= 1.0) */
/*     XNONC : Noncentrality parameter of the chisquare (XNONC >= 0.0) */
// Method
/*     Uses fact that  noncentral chisquare  is  the  sum of a  chisquare */
/*     deviate with k-1  degrees of freedom plus the  square of a normal */
/*     deviate with mean sqrt(XNONC) and standard deviation 1. */
int cdflib_gennch (double k, double xnonc, double *x);

// Generate Non-Central F Random Variables
//
/*     Generates a random deviate from the  noncentral F (variance ratio) */
/*     distribution with v1 degrees of freedom in the numerator, and v2 */
/*     degrees of freedom in the denominator, and noncentrality parameter */
/*     XNONC. */
// Arguments
/*     v1 : Numerator degrees of freedom, v1 >= 1.0 */
/*     v2 : Denominator degrees of freedom (Must be positive) */
/*     XNONC : Noncentrality parameter (Must be nonnegative) */
// Method
/*     Directly generates ratio of noncentral numerator chisquare variate */
/*     to central denominator chisquare variate. */
int cdflib_gennf (double v1, double v2, double xnonc, double *x);

// Generate Multivariate Normal Random Variables
// Arguments
/*     PARM : Parameters needed to generate multivariate normal */
/*               deviates (MEANV and Cholesky decomposition of */
/*               COVM). Set by a previous call to cdflib_setgmn. */
/*               1 : 1                - size of deviate, P */
/*               2 : P + 1            - mean vector */
/*               P+2 : P*(P+3)/2 + 1  - upper half of cholesky */
/*                                       decomposition of cov matrix */
/*               DOUBLE PRECISION PARM(*) */
/*     X    <-- Vector deviate generated. */
/*              DOUBLE PRECISION X(P) */
/*     WORK <: Scratch array */
/*               DOUBLE PRECISION WORK(P) */
// Method
/*     1) Generate P independent standard normal deviates - Ei ~ N(0,1) */
/*     2) Using Cholesky decomposition find A s.t. trans(A)*A = COVM */
/*     3) trans(A)E + MEANV ~ N(MEANV,COVM) */
int cdflib_genmn (double *parm, double *x, double *work);

// SET Generate Multivariate Normal random deviate 
// Function
/*      Places P, MEANV, and the Cholesky factoriztion of COVM */
/*      in PARM for cdflib_genmn. */
// Arguments
/*     MEANV : Mean vector of multivariate normal distribution. */
/*               DOUBLE PRECISION MEANV(P) */
/*     COVM   <: (Input) Covariance   matrix    of  the  multivariate */
/*                 normal distribution.  This routine uses only the */
/*                 (1:P,1:P) slice of COVM, but needs to know LDCOVM. */
/*                 (Output) Destroyed on output */
/*                 DOUBLE PRECISION COVM(LDCOVM,P) */
/*     LDCOVM : Leading actual dimension of COVM. */
/*     P     : Dimension of the normal, or length of MEANV. */
/*     PARM <-- Array of parameters needed to generate multivariate */
/*              normal deviates (P, MEANV and Cholesky decomposition */
/*              of COVM). */
/*              1 : 1                - P */
/*              2 : P + 1            - MEANV */
/*              P+2 : P*(P+3)/2 + 1  - Cholesky decomposition of COVM */
/*              DOUBLE PRECISION PARM(P*(P+3)/2 + 1) */
int cdflib_setgmn (double *meanv, double *covm, int *ldcovm, int *p, double *parm, int *ierr);

// Generate Multinomial Random Variables
// Arguments
/*     N : Number of events that will be classified into one of */
/*           the categories 1..NCAT */
/*     P : Vector of probabilities.  P(i) is the probability that */
/*           an event will be classified into category i.  Thus, P(i) */
/*           must be [0,1]. Only the first NCAT-1 P(i) must be defined */
/*           since P(NCAT) is 1.0 minus the sum of the first */
/*           NCAT-1 P(i). */
/*                         DOUBLE PRECISION P(NCAT-1) */
/*     NCAT : Number of categories.  Length of P and IX. */
/*                         INTEGER NCAT */
/*     IX <-- Observation from multinomial distribution.  All IX(i) */
/*            will be nonnegative and their sum will be N. */
/*                         INTEGER IX(NCAT) */
// Method
/*     Algorithm from page 559 of */
/*     Devroye, Luc */
/*     Non-Uniform Random Variate Generation.  Springer-Verlag, */
/*     New York, 1986. */
int cdflib_genmul (int n, double *p, int ncat, int *ix);

// Generate Random Permutations
/*               GENerate random PeRMutation of xarray */
// Arguments
/*     XARRAY <: On output XARRAY is a random permutation of its */
/*                 value on input */
/*                         DOUBLE PRECISION XARRAY( LARRAY ) */
/*     LARRAY <: Length of XARRAY */
int cdflib_genprm (double *xarray, int larray);

// Generate Negative Binomial Random Variables
// Function
/*     Generates a single random deviate from a negative binomial */
/*     distribution. */
// Arguments
/*     N  : Required number of events. (N > 0) */
/*     P  : The probability of an event during a Bernoulli trial. (0.0 < P < 1.0) */
// Method
/*     Algorithm from page 480 of */
/*     Devroye, Luc */
/*     Non-Uniform Random Variate Generation.  Springer-Verlag, */
/*     New York, 1986. */
int cdflib_ignnbn (int n, double p, int *x);

__END_DECLS


#endif /** _CDFLIB_H_   **/
