// =============================================================================
// Copyright (C) 2012 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// =============================================================================

// <-- JVM NOT MANDATORY -->

path = distfun_getpath (  );
exec(fullfile(path,"tests","unit_tests","testingutilities.sce"));

// Multinomial distribution
// Reference
// http://en.wikipedia.org/wiki/Multinomial_distribution

function [M,C] = mnstats(nb,pr)
    // Multinomial statistics
    //
    // Calling Sequence
    // M = mnstats(nb,pr)
    // [M,C] = mnstats(nb,pr)
    //
    // Parameters
    // nb : a 1-by-1 matrix of doubles, integer value, positive, the number of trials
    // pr : a 1-by-m matrix of doubles, positive, sum to 1, the probability of the m categories
    // M : a 1-by-1 matrix of doubles, the mean
    // C : a m-by-m matrix of doubles, the variance-covariance matrix
    //
    // Description
    // Computes statistics from the multinomial distribution. 
    // On input, we assume that sum(pr)==1.
    //
    // Examples
    // nb = 10
    // pr = [0.3 0.4 0.3]
    // [M,C] = mnstats(nb,pr)
    // Mexpected = [3 4 3]
    // Cexpected = [
	//    2.1  -1.2  -0.9  
	//   -1.2   2.4  -1.2  
	//   -0.9  -1.2   2.1  
	//]
    //
    // Authors
    // Copyright (C) 2012 - Michael Baudin

    M = nb*pr
	m = size(pr,"*")
    C = zeros(m,m)
	for i = 1 : m
		C(i,i) = nb*pr(i)*(1-pr(i))
		for j = i+1 : m
			C(i,j) = -nb*pr(i)*pr(j)
			C(j,i) = C(i,j)
		end
	end
endfunction

function p = mnpdf(x,nb,pr)
    // Multinomial PDF
    //
    // Calling Sequence
    // p = mnpdf(x,nb,pr)
    //
    // Parameters
    // x : a 1-by-m matrix of doubles, integer value, positive, the outcomes
    // nb : a 1-by-1 matrix of doubles, integer value, positive, the number of trials
    // pr : a 1-by-m matrix of doubles, positive, sum to 1, the probability of the m categories
    //
    // Description
    // Computes the PDF from the multinomial distribution. 
    // On input, we assume that sum(pr)==1 and sum(x)==nb.
    //
    // Examples
    // // Compute the probability of the outcome x = [4 0 6]
    // nb = 10
    // pr = [0.3 0.4 0.3]
    // x = [4 0 6]
    // p = mnpdf(x,nb,pr)
    // p_expected = 0.0012400
    //
    // Authors
    // Copyright (C) 2012 - Michael Baudin

    pr = pr(:)
    x = x(:)
    logp = specfun_factoriallog(nb) + sum(x.*log(pr)) - sum(specfun_factoriallog(x))
    p = exp(logp)
endfunction

rtol = 1.e-2;
atol = 1.e-1;

// The number of classes in the histogram
// NC must be even.
NC = 2*12;
N=10000;

// Set the seed to always get the same random numbers
distfun_seedset(0);

// Consider the following experiment. 
// For each experiment, we perform nb trials.
// For each trial, there are 3 outcomes, 
// with probabilities 
// p1=0.2
// p2=0.4
// p3=1-p1-p2=0.4
// Then the random variable R(1:nb,i) in {0,1,2,...,nb} is the 
// number of outcomes in each category, for i=1,2,...,n. 
n = 100000;
nb = 10;
pr = [0.2;0.4];
k = size(pr,"*")+1;
R=grand(n,"mul",nb,pr);
assert_checkequal(size(R),[k n]);
assert_checktrue(R>=0);
assert_checktrue(R<=nb);
assert_checkequal(sum(R,"r"),nb*ones(1,n));
M = mean(R,"c");
C = distfun_cov(R');
p = [pr;1-sum(pr)];
[Mexpected,Cexpected] = mnstats(nb,p);
assert_checkalmostequal(M,Mexpected,1.e-1,[],"element");
assert_checkalmostequal(C,Cexpected,1.e-1,[],"element");
//
// Compute empirical PDF
EmpiricalPDF = [];
for j1 = 0 : nb
    for j2 = 0 : nb
        EmpiricalPDF(j1+1,j2+1) = length(find(R(1,:)==j1 & R(2,:)==j2));
    end
end
EmpiricalPDF = EmpiricalPDF./n;
//
// Compute theoretical PDF
TheoricPDF = zeros(nb+1,nb+1);
for j1 = 0 : nb
    for j2 = 0 : nb
        j3 = max(nb-j1-j2,0);
        if (j1+j2+j3<>nb) then
            continue
        end
        x = [j1 j2 j3]';
        TheoricPDF(j1+1,j2+1) = mnpdf(x,nb,p);
    end
end
//
atol = 0.05*max(TheoricPDF);
assert_checkalmostequal(EmpiricalPDF,TheoricPDF,[],atol,"element");


if (%f) then
    h = scf(); 
    hist3d(EmpiricalPDF);
    hist3d(TheoricPDF);
    h.children.children(1).hiddencolor=2;
    h.children.children(2).hiddencolor=3;
    h.children.rotation_angles = [449 29];
    xtitle("Multinomial Distribution","X1","X2","P(X1,X2)");
end

if (%f) then
    X = 0:nb;
    for j1 = 0:nb
        h = scf();
        plot(X,EmpiricalPDF(j1+1,:),"bo-"); // Empirical Histogram
        plot(X,TheoricPDF(j1+1,:),"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
        xtitle("Distribution of X","X2","P");
        h.children.data_bounds(2,2)=max(TheoricPDF);
    end
end
